//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include "azac_api_cxx_details_property_collection.hpp"
#include "vision_api_cxx_common.hpp"
#include "vision_api_cxx_session_property.hpp"
#include "vision_api_cxx_session_recognizer_property.hpp"

#include <future>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Session {

/// <summary>
/// Represents the properties that can be dynamically updated on a given VisionSession instance
/// </summary>
class VisionSessionProperties : private AI::Core::_detail::PropertyCollectionBase<Session::VisionSessionProperty>
{
private:
    template<typename Target>
    using ProtectedAccess = Azure::AI::Core::_detail::ProtectedAccess<Target>;
    using PropertyCollection = AI::Core::_detail::PropertyCollectionBase<Session::VisionSessionProperty>;

public:
    using PropertyCollection::Set;
    using PropertyCollection::Get;

private:

    friend class Azure::AI::Core::_detail::ProtectedAccess<VisionSessionProperties>;
    friend class Azure::AI::Core::_detail::PropertyCollection<Azure::AI::Vision::Core::Session::VisionSessionProperty, Azure::AI::Vision::Core::Session::VisionSessionProperties>;
    friend class Azure::AI::Core::_detail::PropertyCollection<Azure::AI::Vision::Core::Session::VisionSessionRecognizerProperty, Azure::AI::Vision::Core::Session::VisionSessionProperties>;

    VisionSessionProperties(AZAC_HANDLE handle) : PropertyCollection(handle) {}
    explicit operator AZAC_HANDLE() { return ProtectedAccess<PropertyCollection>::HandleFromPtr(this); }
};

} } } } } // Azure::AI::Vision::Core::Session
