//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include "vision_api_cxx_common.hpp"
#include "vision_api_cxx_details_session_event_args_base.hpp"

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Session {

/// <summary>
/// Represents inference processing starting.
/// </summary>
class SessionStartedEventArgs : private Core::_detail::SessionEventArgsBase<int>
{
private:

    using BaseEventArgs = Core::_detail::SessionEventArgsBase<int>;

public:

    /// <summary>
    /// Gets the unique id for the Session from which this SessionResult originated.
    /// </summary>
    /// <returns>
    /// The Session ID string.
    /// </returns>
    std::string GetSessionId() const { return BaseEventArgs::GetSessionId(); }

    /// <summary>
    /// Gets the unique id for the Session from which this SessionResult originated.
    /// </summary>
    /// <returns>
    /// The Session ID string.
    /// </returns>
    template<class T = std::string> 
    AI::Core::_detail::enable_if_w_or_string_t<T> GetSessionId() const { return BaseEventArgs::GetSessionId<T>(); }

private:

    friend class Azure::AI::Core::_detail::ProtectedAccess<SessionStartedEventArgs>;

    /*! \cond PROTECTED */
    static std::shared_ptr<SessionStartedEventArgs> FromHandle(AZAC_HANDLE handle)
    {
        auto ptr = new SessionStartedEventArgs(handle);
        return std::shared_ptr<SessionStartedEventArgs>(ptr);
    }

    explicit SessionStartedEventArgs(AZAC_HANDLE eventArgs) : SessionEventArgsBase(eventArgs)
    {
    }
    /*! \endcond */

    AZAC_DISABLE_DEFAULT_CTORS(SessionStartedEventArgs);
};

} } } } } // Azure::AI::Vision::Core::Session
